import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';

void main() {
  runApp(MyApp());
}

class MyApp extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    String title = "";
    return MaterialApp(
      title: 'lq demo 0519',
      theme: ThemeData(
        primarySwatch: Colors.yellow,
      ),
      home: BasicMessageChannelPage(title),
    );
  }
}

class BasicMessageChannelPage extends StatefulWidget {
  String title;
  BasicMessageChannelPage(this.title);

  @override
  _BasicMessageChannelPageState createState() =>
      _BasicMessageChannelPageState(title);
}

class _BasicMessageChannelPageState extends State<BasicMessageChannelPage> {
  String title;

  _BasicMessageChannelPageState(this.title);

  String receive = "null";
  static const messageChannel =
      const BasicMessageChannel("com.sicilyliu", StandardMessageCodec());

  // 发送消息
  Future<dynamic> sendMessage(Map json) async {
    if (json["method"] == "battery") {
      int battery = await messageChannel.send(json);
      setState(() {
        receive = "i am sicilyliu, i find the battery = $battery";
      });
      return battery;
    } else {
      Map reply = await messageChannel.send(json);

      // 解析参数
      int code = reply['code'];
      String message = reply['message'];

      setState(() {
        receive = "flutter send to native  code : $code, message: $message";
      });
      return reply;
    }
  }

  // 消息监听
  void receiveMessage() {
    messageChannel.setMessageHandler((message) async {
      // 解析参数
      int code = message['code'];
      String message1 = message['message'];

      setState(() {
        receive = "native send to flutter  code : $code, message: $message1";
      });
      return 'flutter receive message';
    });
  }

  @override
  void initState() {
    super.initState();
    receiveMessage();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: new AppBar(
        backgroundColor: Colors.green,
        centerTitle: true,
        leading: Container(),
        title: new Text(title),
      ),
      body: buildWidget(context),
    );
  }

  buildWidget(BuildContext context) {
    return Column(
      mainAxisSize: MainAxisSize.max,
      children: <Widget>[
        Padding(
          padding: EdgeInsets.all(10),
          child: FlatButton(
            color: Colors.purple,
            onPressed: () {
              sendMessage(
                  {"method": "hello", "ontent": "hello的数据", "code": 100});
            },
            child: Text(
              "hello",
              style: TextStyle(color: Colors.white),
            ),
          ),
        ),
        Padding(
          padding: EdgeInsets.all(10),
          child: FlatButton(
            color: Colors.purple,
            onPressed: () {
              sendMessage(
                  {"method": "battery", "ontent": "battery电量", "code": 200})
                ..then((value) {});
            },
            child: Text(
              "battery",
              style: TextStyle(color: Colors.white),
            ),
          ),
        ),
        Padding(
          padding: EdgeInsets.all(10),
          child: FlatButton(
            color: Colors.purple,
            onPressed: () {
              sendMessage(
                  {"method": "getTime", "ontent": "getTime时间", "code": 300});
            },
            child: Text(
              "getTime",
              style: TextStyle(color: Colors.white),
            ),
          ),
        ),
        Padding(
          padding: EdgeInsets.all(10),
          child: new Text(
            "$receive",
            style: TextStyle(
              fontSize: 20,
              color: Colors.blue,
            ),
          ),
        ),
      ],
    );
  }
}
